
function [ x, fx, flag, output ] = NelderMead( f, x0, options )
    % f         function handle
    % x0        initial guess - each row is a n-dimensional point
    % options   optimset options (TolX, TolF, MaxIter, MaxFunEvals)

    if isempty(options.TolX)
        options.TolX = 1e-3;
    end
    if isempty(options.TolFun)
        options.TolFun = 1e-3;
    end
    if isempty(options.MaxIter)
        options.MaxIter = 1e3;
    end
    if isempty(options.MaxFunEvals)
        options.MaxFunEvals = 1e3;
    end
    
    tolX = options.TolX;
    tolF = options.TolFun;
    maxIter = options.MaxIter;
    maxFunEval = options.MaxFunEvals;

    rho = 1; chi = 2; gamma = 0.5; sigma = 0.5;

    n       = size(x0,2);
    two2np1 = 2:n+1;
    one2n   = 1:n;
    
    % initial simplex
    if size(x0,1)==1
        x = zeros(n+1,n);
        x(1,:) = x0;
        for j=1:n
            x(j+1,:) = x0;
            if x0(j)~=0
                x(j+1,j) = 1.05*x0(j);
            else
                x(j+1,j) = 0.00025;
            end
        end
    else
        x = x0;
    end
    %x0     = x;
    
    fx     = evalF(f,x);
    [fx,j] = sort(fx);
    x      = x(j,:);
    x_bar  = sum(x(one2n,:))/n;
    
    flag         = 0;
    iterCount    = 1;
    funEvalCount = n+1;
    while iterCount<maxIter && funEvalCount<maxFunEval 
        if max(abs(fx(1)-fx(two2np1)))<=tolF && ...
                max(max(abs(x(two2np1,:)-x(ones(1,n),:))))<=tolX
            flag = 1; % algorithm has converged
            break
        end

        %plotF(f,x0,x,fx);

        shrink = 0;
        x_r = (1+rho)*x_bar - rho*x(n+1,:);
        fx_r = f(x_r);
        funEvalCount = funEvalCount+1;
        if fx_r<fx(1)
            x_e = (1+rho*chi)*x_bar - rho*chi*x(n+1,:);
            fx_e = f(x_e);
            funEvalCount = funEvalCount+1;
            if fx_e<fx_r
                %op = 'expand';
                x(n+1,:) = x_e;
                fx(n+1) = fx_e;
            else
                %op = 'reflect';
                x(n+1,:) = x_r;
                fx(n+1) = fx_r;
            end
        else
            if fx_r<fx(n)
                %op = 'reflect';
                x(n+1,:) = x_r;
                fx(n+1) = fx_r;
            else
                if fx_r<fx(n+1)
                    x_c = (1+rho*gamma)*x_bar - rho*gamma*x(n+1,:);
                    fx_c = f(x_c);
                    funEvalCount = funEvalCount+1;
                    if fx_c<=fx_r
                        %op = 'contract outside';
                        x(n+1,:) = x_c;
                        fx(n+1) = fx_c;
                    else
                        %op = 'shrink';
                        shrink = 1;
                    end
                else
                    x_c = (1-gamma)*x_bar + gamma*x(n+1,:);
                    fx_c = f(x_c);
                    funEvalCount = funEvalCount+1;
                    if fx_c<fx(n+1)
                        %op = 'contract inside';
                        x(n+1,:) = x_c;
                        fx(n+1) = fx_c;
                    else
                        %op = 'shrink';
                        shrink = 1;
                    end
                end
            end
        end
        if shrink
            x(two2np1,:) = x(ones(1,n),:) + sigma*(x(two2np1,:)-x(ones(1,n),:));
            fx(two2np1) = evalF(f,x(two2np1,:));
            funEvalCount = funEvalCount+n;
            % normal update
            [fx,j] = sort(fx);
            x = x(j,:);
            x_bar = sum(x(one2n,:))/n;
        else
            % efficient update
            jj = n;
            while jj>=1 && fx(jj+1)<fx(jj)
                temp = fx(jj);
                fx(jj) = fx(jj+1);
                fx(jj+1) = temp;
                temp = x(jj,:);
                x(jj,:) = x(jj+1,:);
                x(jj+1,:) = temp;
                jj = jj-1;
            end
            if jj<n
                x_bar = x_bar+x(jj+1,:)/n-x(n+1,:)/n;
            end
        end
        %fprintf('%03d >> %.20f %s \n',c,fx(1),op)
        iterCount = iterCount + 1;
    end
    x  = x(1,:);
    fx = fx(1);
    output.iterations = iterCount;
    output.funcCount = funEvalCount;
end

function fx = evalF(f, x)
    fx = zeros(size(x,1),1);
    for i=1:size(x,1)
        fx(i) = f(x(i,:));
    end
end

function plotF(f, x0, x, fx)
    if size(x0,2)==2
        x_min = min(min(x(:,1)),min(x0(:,1)));
        x_max = max(max(x(:,1)),max(x0(:,1)));
        y_min = min(min(x(:,2)),min(x0(:,2)));
        y_max = max(max(x(:,2)),max(x0(:,2)));
        xx = x_min:(x_max-x_min)/50:x_max;
        yy = y_min:(y_max-y_min)/50:y_max;
        [X,Y] = meshgrid(xx,yy);
        fXY = evalF(f,[X(:) Y(:)]);
        h = figure(1);
        clf(h);
        hold on
        mesh(X,Y,reshape(fXY,size(X)),'EdgeColor','k','FaceColor','none','EdgeAlpha',0.25)
        plot3(x(:,1),x(:,2),fx,'or')
        plot3([x(:,1); x(1,1)],[x(:,2); x(1,2)],[fx; fx(1)],'-r')
        view([25 65])
        hold off
        box on
        pause(0.1)
    elseif size(x0,2)==1
        x_min = min(min(x),min(x0));
        x_max = max(max(x),max(x0));
        xx = (x_min:(x_max-x_min)/50:x_max)';
        fxx = evalF(f,xx);
        h = figure(1);
        clf(h);
        hold on
        plot(xx,fxx)
        plot(x,fx,'or')
        plot([x; x(1)],[fx; fx(1)],'-r')
        hold off
        pause(0.3)
    end
end

